/*! \file *********************************************************************
 *
 * \brief
 *      Realisierung eines Takmanagers
 * 
 * 		Abarbeitung der Tasklisten
 * 		Hinzufgen und Lschen von Task in/aus den Listen 
 *
 * \par Projekt:
 *      Beatmungsgeraet - Bediencontroller - TRENDvent \n
 *      Prozessor: ATmega2560 \n
 *		Frequenz: 14.7456 MHz
 *
 * \author 
 *      Hoffrichter GmbH: http://www.hoffrichter.com \n
 *      Support email: info@hoffrichter.de
 *
 * Name: Steffen Graap
 * Date: 16.12.2008  \n
 ******************************************************************************/

#include "_task.h"
#include <avr/interrupt.h>
#include <stddef.h>
#include "_main.h"

//Variablen
TIMER_RESOLUTION tTimeDelay;				//Delay bis zum nchsten auszufhrenden Zeittask
TIMER_RESOLUTION tTimeDelayOffset;			//Offset der entsteht, wenn Abarbeitung in der Programmschleife verzgert wird
UINT8 u8NextTimeEntry;						//Index des nchsten auszufhrenden Zeittask
UINT8 u8NextPrioEntry;						//Index des Task mit der hchsten Prioritt

struct _TimeElement aTimeList [MAX_TIME_ENTRYS];//!< Auftragsliste fr Zeittasks
struct _PrioElement aPrioList [MAX_PRIO_ENTRYS];//!< Auftragsliste fr Prioritentasks

//****************************************************************************
/*! \brief Funktion: vInit_Time_Task\n
 * 		- Initialisierung der Time-Task-Liste
 * 		\n\n 
 * erstellt: 17.12.2008
 */
 //*****************************************************************************
void vInit_Time_Task (void)
{
UINT8 u8Counter;
UINT8 u8SREG_Save = 1;
for (u8Counter = MAX_TIME_ENTRYS ; u8Counter ; u8Counter--)//fr alle Listeneintrge
	{
	aTimeList [u8Counter-1].u8NextIndex = T_FREE;//als frei markieren
	aTimeList [u8Counter-1].tT_Delta = 0;//(TIMER_RESOLUTION)0 - 1;//Delay mit Maximalwert initialisieren
	aTimeList [u8Counter-1].pFunktion = NULL_P;
	u8NextTimeEntry = T_LAST; 				//kiene Task vorhanden
	u8SREG_Save = SREG & (1 << SREG_I);
	cli ();
	tTimeDelay = 0;
	if (u8SREG_Save) sei ();
	}
}

//****************************************************************************
/*! \brief Funktion: u8Add_Prio_Task\n
 * 		- Task in Prioritenliste einfgen
 * 		- Suche der Position nach Prioritt in der Liste
 * 		- Aktualiserung der Vekettungsvariablen  \n\n 
 * erstellt: 18.12.2008
 * 
 * \param pFunktion Pointer auf die Funktion die ausgefhrt werden soll
 * \param u8Prioritaet Prioritt mit der Funktion ausgefhrt werden soll
 * 
 * \return UINT8
 * 
 * \retval Status der Aktion
 * 		- TRUE = Eintrag wurde erfolgreich erstellt
 * 		- FASLE = Liste voll
 */
 //*****************************************************************************
UINT8 u8Add_Prio_Task (funcp pFunktion , UINT8 u8Prioritaet)
{
UINT8 u8Counter;
UINT8 u8Index; 								//Index des aktuellen Eintrages
UINT8 u8PreIndex = 0; 						//Index des vorherigen Eintrages
UINT8 u8LastPrio;							//Prioritt des aktuellen Eintrages
struct _PrioElement * pPointer; 			//Pointer fr schnelleren Zugriff

//******************** berprfen, ob Task nicht schon existiert ************************
if (u8Check_Prio_Task (pFunktion)) u8Remove_Prio_Task (pFunktion);
//******************** finden eines freien Listenplatzes ************************
for (u8Counter = 0;;)
	{
	pPointer = & aPrioList[u8Counter];		//Pointer auf aktuellen Listeneintrag legen 
	if (pPointer->u8NextIndex == T_FREE) break;//wenn freier Eintrag gefunden, Schleife abbrechen
	u8Counter++;							//wenn Listeneintrag nicht frei, nchster Listeneintrag
	if (u8Counter == MAX_PRIO_ENTRYS)
		{
#if DEBUG_MODE
		u8MaxPrioTaskEntrys = MAX_PRIO_ENTRYS;
#endif
		return 1;//wenn Ende der Liste erreicht, Funktion abbrechen und Error als Rckgabewert zurckgeben
		}
	}
#if DEBUG_MODE
if (u8Counter > u8MaxPrioTaskEntrys) u8MaxPrioTaskEntrys = u8Counter;//TODO fr Debugzwecke
#endif
//******************** Suche der Position nach Prioritt in die Liste *****************
u8Index = u8NextPrioEntry;					//Index des Hchstprioren Tasks laden
while (u8Index != T_LAST )					//solange der letzte Eintrag noch nicht gefunden
	{
	u8LastPrio = aPrioList [u8Index].u8Prioritaet;//Prioritt laden
	if (u8LastPrio > u8Prioritaet)			//wenn neuer Eintrag hhere Prioritt als der aktuell aus der Liste geladene Eintrag
		{
		break;								//Abbruch der Suche, da Position nach Prioritt gefunden wurde
		}
	else									//wenn neuer Eintrag spter ausgefhrt werden soll als der aktuell aus der Liste geladene Eintrag
		{
		u8PreIndex = u8Index;				//vorherigen Eintrag fr eventuelle Verkettung merken
		u8Index = aPrioList [u8Index].u8NextIndex;//Index des nchsten Eintrages aus Verkettungseintrag laden
		}
	}
//******************** Eintrag an priorer Position in die Liste *****************
pPointer->u8NextIndex = u8Index;			//Index des nchsten Eintrages speichern
pPointer->u8Prioritaet = u8Prioritaet;		//Prioritt des Eintrags speichern
pPointer->pFunktion = pFunktion;			//Pointer auf die auszufhrenden Funktion speichern
//******************** Verkettungen aktualisieren *****************
if (u8Index == u8NextPrioEntry)				//wenn neuer Eintrag der nchste auszufhrende ist
	{
	u8NextPrioEntry = u8Counter;			//globalen Index sichern
	}
else										//wenn neuer Eintrag nicht der nchste auszufhrende ist
	{
	aPrioList [u8PreIndex].u8NextIndex = u8Counter;//Verkettungsindex aktualisieren
	}
return 0;									//Rckgabewert fr OK zurckgeben
}

//****************************************************************************
/*! \brief Funktion: u8Remove_Prio_Task\n
 * 		- Entfernen eines Tasks aus der Prioritten liste
 * 		- Suche nach der Funktion , die entfernt werden soll
 * 		- Aktualisierung der erforderlichen Verkettungsvarianblen
 * 		n\n 
 * erstellt: 18.12.2008
 * 
  \param pFunktion Pointer suf die Funktion die aus der Liste entfernt werden soll
 * 
 * \return UINT8
 * 
 * \retval Status der Aktion
 * 		- TRUE = Eintrag wurde erfolgreich entfernt
 * 		- FASLE = Funktion nicht gefunden
 */
 //*****************************************************************************
UINT8 u8Remove_Prio_Task (funcp pFunktion)
{
UINT8 u8RemIndex = 0;						//Index des aktuellen Eintrages
UINT8 u8PreIndex;	 						//Index des vorherigen Eintrages
UINT8 u8FolIndex = u8NextPrioEntry;			//Index des nchsten Eintrages mit zeilich nchsten Eintrag laden
struct _PrioElement * pPointer; 			//Pointer fr schnelleren Zugriff
//******************** nach Eintrag anhand der Funktion suchen ****************
do	{
	if (u8FolIndex == T_LAST)				//wenn nchster Eintrag der letzte ist
		return 1; 							//Abbruch der suche und Rckgabe eines Fehlers
	u8PreIndex = u8RemIndex;				//Indexe aktualisieren
	u8RemIndex = u8FolIndex;
	pPointer = & aPrioList [u8RemIndex];	//Pointer mit aktuellen Listenzeile laden
	u8FolIndex = pPointer->u8NextIndex;		//Index des nchsten laden
	}
while (pPointer->pFunktion != pFunktion );	//solange wiederholen bis Eintrag (Funktion) gefunden wurde
//************************* Eintrag lschen **********************************
pPointer->u8NextIndex = T_FREE;				//gefundenen Eintrag als frei markieren
pPointer->u8Prioritaet = 255;				//Prioritt mit minimaler Prioritt initialisieren
pPointer->pFunktion = NULL_P;
//************************ Verkettung aktualisieren *****************************
if (u8RemIndex == u8NextPrioEntry)			//wenn gelschter Eintrag der nchste auszufhrende ist
	{
	u8NextPrioEntry = u8FolIndex;			//aktuellen Index aktualisieren
	}
else										//wenn gelschter Eintrag nicht der nchste auszufhrende ist
	{
	aPrioList [u8PreIndex].u8NextIndex = u8FolIndex;//Verkettung aktualisieren
	}
return 0;									//Rckgabewert fr OK zurckgeben
}

//****************************************************************************
/*! \brief Funktion: u8Check_Prio_Task \n 
 * 		- berprft ob ein Prio-Task bereits geplant ist
 * 		\n\n 
 * erstellt: 03.02.2009
 * 
 * \param pFunktion Pointer auf die Funktion die in der Liste gesucht werden soll
 * 
 * \return UINT8
 * 
 * \retval Status der Aktion
 * 		- TRUE = Funktion gefunden
 * 		- FASLE = Funktion nicht gefunden
 */
 //*****************************************************************************
UINT8 u8Check_Prio_Task (funcp pFunktion)
{
UINT8 u8RemIndex = 0;						//Index des aktuellen Eintrages
UINT8 u8PreIndex;	 						//Index des vorherigen Eintrages
UINT8 u8FolIndex = u8NextPrioEntry;			//Index des nchsten Eintrages mit zeilich nchsten Eintrag laden
struct _PrioElement * pPointer; 			//Pointer fr schnelleren Zugriff
do	{
	if (u8FolIndex == T_LAST)				//wenn nchster Eintrag der letzte ist
		return 0; 							//Abbruch der Suche und Rckgabe fr Funktion nicht gefunden
	u8PreIndex = u8RemIndex;				//Indexe aktualisieren
	u8RemIndex = u8FolIndex;
	pPointer = & aPrioList [u8RemIndex];	//Pointer mit aktuellen Listenzeile laden
	u8FolIndex = pPointer->u8NextIndex;		//Index des nchsten laden
	}
while (pPointer->pFunktion != pFunktion );	//solange wiederholen bis Eintrag (Funktion) gefunden wurde
return 1;									//Rckgabewert fr Funktion nicht gefunden zurckgeben
}

//****************************************************************************
/*! \brief Funktion: vInit_Prio_Task\n
 * 		- Initialisierung der Prioritten-Task-Liste
 * 		\n\n 
 * erstellt: 17.12.2008
 */
 //*****************************************************************************
void vInit_Prio_Task (void)
{
UINT8 u8Counter;

for (u8Counter = MAX_PRIO_ENTRYS ; u8Counter ; u8Counter--)//fr alle Listeneintrge
	{
	aPrioList [u8Counter-1].u8NextIndex = T_FREE;//als frei markieren
	aPrioList [u8Counter-1].u8Prioritaet = 255;//Prioritt mit minimaler Prioritt initialisieren
	aPrioList [u8Counter-1].pFunktion = NULL_P;
	u8NextPrioEntry = T_LAST;				//kiene Task vorhanden
	}
}

//****************************************************************************
/*! \brief Funktion: vTaskmanager\n
 * 		- Taskmanager 
 * 		- Abarbeitung eines zeitlichen Tasks, wenn vorhanden und Delay abgelaufen
 * 		- Abbarbeitung eines Priorittentasks, wenn vorhanden
 * 		- Freigabe der Interrupts unbedingt vor Funktionsaufruf
 * 		\n\n 
 * erstellt: 17.12.2008
 */
 //*****************************************************************************
void vTaskmanager (void)
{
struct _TimeElement * pTime_Pointer; 		//!< Pointer fr schnelleren Zugriff
struct _PrioElement * pPrio_Pointer; 		//!< Pointer fr schnelleren Zugriff
funcp pLokal_Funktion;
UINT8 u8SREG_Save = 1;
//********************************* Zeittasks *********************************
u8SREG_Save = SREG & (1 << SREG_I);
cli ();										//!!! ACHTUNG Interrupts mssen unbedingt vor Funktionsaufruf wieder frei gegeben werden !!!!!!!!!
if (tTimeDelay == 0)						//wenn Delay abgelaufen
	{
	if (u8NextTimeEntry != T_LAST)			//und Tasks vorhanden
		{
		pTime_Pointer = & aTimeList [u8NextTimeEntry];//Pointer fr Zugriff laden
		u8NextTimeEntry = pTime_Pointer->u8NextIndex;//nchsten Index in globale Variable laden
		tTimeDelay = aTimeList [u8NextTimeEntry].tT_Delta;//nchstes Delay laden
		if (tTimeDelayOffset)				//wenn eine Verzgerung aufgetreten ist
			{
			if (tTimeDelayOffset >= tTimeDelay)//und die Verzgerung grer als nchstes Delay
				{
				tTimeDelayOffset -=tTimeDelay;//Verzgerung fr bernchstes Delay berechnen
				tTimeDelay = 0;				//Delay auf 0 seten
				}
			else							//wenn Verzgerung kleiner als nchstes Delay
				{
				tTimeDelay -= tTimeDelayOffset;//Delay berechnen
				tTimeDelayOffset = 0;		//Verzgerung zurcksetzen
				}
			}
		if (u8SREG_Save) sei ();			//hier schon Freigeben, damit Funktionsaufruf nicht zur Verzgerung fhrt
		pLokal_Funktion = pTime_Pointer->pFunktion;//Funktionspointer in lokale Variable bernehmen
		//Eintrag vor Aufruf lschen, damit ggf. durch die aufgerufenen Funktion der Eintrag neu erstllt werden kann
		pTime_Pointer->u8NextIndex = T_FREE;//Eintrag als frei markieren
		pTime_Pointer->tT_Delta = (TIMER_RESOLUTION)0 - 1;//Delay auf max setzen
		pTime_Pointer->pFunktion = NULL_P;	//Funktionspointer lschen
		(* pLokal_Funktion) ();				//auszufhrende Funktion aufrufen
		}
	else{
		tTimeDelayOffset = 0;				//wenn keine Tasks vorhanden, Verzgerung zurcksetzen
		}
	}
if (u8SREG_Save) sei ();
//********************************* Priorittentasks *********************************
if (u8NextPrioEntry != T_LAST)				//sind Tasks vorhanden
	{
	pPrio_Pointer = & aPrioList [u8NextPrioEntry];//Pointer fr Zugriff laden
	u8NextPrioEntry = pPrio_Pointer->u8NextIndex;//nchsten Index in globale Variable laden
	pPrio_Pointer->pFunktion ();			//auszufhrende Funktion aufrufen
	pPrio_Pointer->u8NextIndex = T_FREE;	//eintrag als frei markieren
	pPrio_Pointer->u8Prioritaet = 255;		//Prioritt mit minimaer Prioritt initialisieren
	pPrio_Pointer->pFunktion = NULL_P;		//Funktionspointer lschen
	}
}

//****************************************************************************
/*! \brief Funktion: u8Add_Time_Task\n
 * 		- Task in die Zeitliste eintragen
 * 		- Suche der zeitlichen Position in der Liste
 * 		- Aktualisierung der erforderlichen Verkettungsvarianblen 
 * 		\n\n 
 * erstellt: 16.12.2008
 * 
 * \param pFunktion Pointer auf die Funktion die ausgefhrt werden soll
 * \param tDelay Zeit die bis zum ausfhren der Funktion verstreichen soll
 * 
 * \return UINT8
 * 
 * \retval Status der Aktion
 * 		- TRUE = Eintrag wurde erfolgreich erstellt
 * 		- FASLE = Liste voll
 */
 //*****************************************************************************
UINT8 u8Add_Time_Task (funcp pFunktion , TIMER_RESOLUTION tDelay)
{
UINT8 u8Counter;
UINT8 u8Index; 								//Index des aktuellen Eintrages
UINT8 u8PreIndex = 0; 						//Index des vorherigen Eintrages
TIMER_RESOLUTION tLastDelay; 				//Delay des akktuellen Task
struct _TimeElement * pPointer; 			//Pointer fr schnelleren Zugriff
UINT8 u8SREG_Save = 1;
//******************** berprfen, ob Task nicht schon existiert ************************
if (u8Check_Time_Task (pFunktion)) u8Remove_Time_Task (pFunktion);
//******************** finden eines freien Listenplatzes ************************
for (u8Counter = 0;;)
	{
	pPointer = & aTimeList[u8Counter];		//Pointer auf aktuellen Listeneintrag legen 
	if (pPointer->u8NextIndex == T_FREE) break;//wenn freier Eintrag gefunden, Schleife abbrechen
	u8Counter++;							//wenn Listeneintrag nicht frei, nchster Listeneintrag
	if (u8Counter == MAX_TIME_ENTRYS)
		{
#if DEBUG_MODE
		u8MaxTimeTaskEntrys = MAX_TIME_ENTRYS;
#endif
		return 1;//wenn Ende der Liste erreicht, Funktion abbrechen und Error als Rckgabewert zurckgeben
		}
	}
#if DEBUG_MODE
if (u8Counter > u8MaxTimeTaskEntrys) u8MaxTimeTaskEntrys = u8Counter;//TODO fr Debugzwecke
#endif
//******************** Suche der zeitliche Position in die Liste *****************
u8Index = u8NextTimeEntry;					//Index des zeitlich nchsten Tasks laden
u8SREG_Save = SREG & (1 << SREG_I);
cli ();
tLastDelay = tTimeDelay;					//Delay des zeitlich nchsten Task laden
if (u8SREG_Save) sei ();					//Interruptes wieder freigeben
while (u8Index != T_LAST )					//solange der letzte Eintrag noch nicht gefunden
	{
	if (tLastDelay >= tDelay)				//wenn neuer Eintrag frher ausgefhrt werden soll als der aktuell aus der Liste geladene Eintrag
		{
		aTimeList [u8Index].tT_Delta = tLastDelay - tDelay;//Delay fr bisher frhste Eintrag neu berechnen und sichern 
		break;								//da zeitliche Position gefunden wurde , Abbruch der Suche
		}
	else									//wenn neuer Eintrag spter ausgefhrt werden soll als der aktuell aus der Liste geladene Eintrag
		{
		tDelay -= tLastDelay; 				//Restdelay zum aktuellen Eintrag berechnen
		u8PreIndex = u8Index;				//vorherigen Eintrag fr eventuelle Verkettung merken
		u8Index = aTimeList [u8Index].u8NextIndex;//Index des nchsten Eintrages aus Verkettungseintrag laden
		tLastDelay = aTimeList [u8Index].tT_Delta;//Delayoffset zum zeitlich frheren laden 
		}
	}
//******************** Eintrag an zeitliche Position in die Liste *****************
pPointer->u8NextIndex = u8Index;			//Index des nchsten eintrages speichern
pPointer->tT_Delta = tDelay;				//Delay zum vorherigen Eintrag speichern
pPointer->pFunktion = pFunktion;			//Pointer auf die auszufhrenden Funktion speichern
//******************** Verkettungen aktualisieren *****************
if (u8Index == u8NextTimeEntry)				//wenn neuer Eintrag der nchste auszufhrende ist
	{
	u8NextTimeEntry = u8Counter;			//globalen Index sichern
	u8SREG_Save = SREG & (1 << SREG_I);
	cli ();									//Interrupts abschalten , damit keine Vernderung der Delayvariablen
	tTimeDelay = tDelay;					//globales Delay sichern
	if (tTimeDelayOffset)					//wenn eine Verzgerung aufgetreten ist
		{
		if (tTimeDelayOffset >= tTimeDelay)	//und die Verzgerung grer als nchstes Delay
			{
			tTimeDelayOffset -=tTimeDelay;	//Verzgerung fr bernchstes Delay berechnen
			tTimeDelay = 0;					//Delay auf 0 seten
			}
		else								//wenn Verzgerung kleiner als nchstes Delay
			{
			tTimeDelay -= tTimeDelayOffset;	//Delay berechnen
			tTimeDelayOffset = 0;			//Verzgerung zurcksetzen
			}
		}
	if (u8SREG_Save) sei ();				//Interruptes wieder freigeben
	}
else{										//wenn neuer Eintrag nicht der nchste auszufhrende ist
	aTimeList [u8PreIndex].u8NextIndex = u8Counter;//Verkettungsindex aktualisieren
	}
return 0;									//Rckgabewert fr OK zurckgeben
}

//****************************************************************************
/*! \brief Funktion: u8Remove_Time_Task \n
 * 		- Task aus Zeitliste entfernen
 * 		- Suche nach der Funktion , die entfernt werden soll
 * 		- Aktualisierung der erforderlichen Verkettungsvarianblen 
 * 		\n\n 
 * erstellt: 18.12.2008
 * 
 * \param pFunktion Pointer auf die Funktion die aus der Liste entfernt werden soll
 * 
 * \return UINT8
 * 
 * \retval Status der Aktion
 * 		- TRUE = Eintrag wurde erfolgreich entfernt
 * 		- FASLE = Funktion nicht gefunden
 */
 //*****************************************************************************
UINT8 u8Remove_Time_Task (funcp pFunktion)
{
UINT8 u8RemIndex = 0;						//Index des aktuellen Eintrages
UINT8 u8PreIndex;	 						//Index des vorherigen Eintrages
UINT8 u8FolIndex = u8NextTimeEntry;			//Index des nchsten Eintrages mit zeilich nchsten Eintrag laden
struct _TimeElement * pPointer; 			//Pointer fr schnelleren Zugriff
UINT8 u8SREG_Save = 1;
//******************** nach Eintrag anhand der Funktion suchen ****************
do	{
	if (u8FolIndex == T_LAST)				//wenn nchster Eintrag der letzte ist
		return 1; 							//Abbruch der Suche und Rckgabe eines Fehlers
	u8PreIndex = u8RemIndex;				//Indexe aktualisieren
	u8RemIndex = u8FolIndex;
	pPointer = & aTimeList [u8RemIndex];	//Pointer mit aktuellen Listenzeile laden
	u8FolIndex = pPointer->u8NextIndex;		//Index des nchsten laden
	}
while (pPointer->pFunktion != pFunktion );	//solange wiederholen bis Eintrag (Funktion) gefunden wurde
//************************ Verkettung aktualisieren *****************************
if (u8RemIndex == u8NextTimeEntry)			//wenn gelschter Eintrag der nchste auszufhrende ist
	{
	u8NextTimeEntry = u8FolIndex;			//aktuellen Index aktualisieren
	u8SREG_Save = SREG & (1 << SREG_I);
	cli ();
	tTimeDelay += aTimeList [u8FolIndex].tT_Delta;//aktuelles Delay aktualiseren
	if (u8SREG_Save) sei ();
	}
else										//wenn gelschter Eintrag nicht der nchste auszufhrende ist
	{
	aTimeList [u8PreIndex].u8NextIndex = u8FolIndex;//Verkettung aktualisieren
	if (u8FolIndex != T_LAST)				//wenn zu entfernender Eintrag nicht der letzte war
		{
		aTimeList[u8FolIndex].tT_Delta += pPointer->tT_Delta;//Delay des zu entfernenden Eintrages auf das Delay des darauf folgenden aufschlagen  
		}
	}
//************************* Eintrag lschen **********************************
pPointer->u8NextIndex = T_FREE;				//gefundenen Eintrag als frei markieren
pPointer->tT_Delta = (TIMER_RESOLUTION)0 - 1;//Delay mit Maximalwert initialisieren
pPointer->pFunktion = NULL_P;
return 0;									//Rckgabewert fr OK zurckgeben
}

//****************************************************************************
/*! \brief Funktion: u8Check_Time_Task \n 
 * 		- berprft ob ein Time-Task bereits geplant ist
 * 		 \n\n 
 * erstellt: 03.02.2009
 * 
 * \param pFunktion Pointer auf die Funktion die in der Liste gesucht werden soll
 * 
 * \return UINT8
 * 
 * \retval 0 = Funktion nicht gefunden , != 0 = Indexplatz der Funktion
 */
 //*****************************************************************************
UINT8 u8Check_Time_Task (funcp pFunktion)
{
UINT8 u8RemIndex = 0;						//Index des aktuellen Eintrages
UINT8 u8PreIndex;	 						//Index des vorherigen Eintrages
UINT8 u8FolIndex = u8NextTimeEntry;			//Index des nchsten Eintrages mit zeilich nchsten Eintrag laden
struct _TimeElement * pPointer; 			//Pointer fr schnelleren Zugriff
do	{
	if (u8FolIndex == T_LAST)				//wenn nchster Eintrag der letzte ist
		return 0; 							//Abbruch der Suche und Rckgabe fr Funktion gefunden
	u8PreIndex = u8RemIndex;				//Indexe aktualisieren
	u8RemIndex = u8FolIndex;
	pPointer = & aTimeList [u8RemIndex];	//Pointer mit aktuellen Listenzeile laden
	u8FolIndex = pPointer->u8NextIndex;		//Index des nchsten laden
	}
while (pPointer->pFunktion != pFunktion );	//solange wiederholen bis Eintrag (Funktion) gefunden wurde
return u8RemIndex;							//Rckgabewert fr Funktion gefunden zurckgeben
}

